@@ -22,23 +22,30 @@ BitBake build tools.
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, re, sys
+import os, sys
+import warnings
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
+import logging
+import atexit
+import traceback
 import bb.utils
-import pickle
 
 # This is the pid for which we should generate the event. This is set when
 # the runqueue forks off.
 worker_pid = 0
 worker_pipe = None
 
-class Event:
+class Event(object):
     """Base class for events"""
 
     def __init__(self):
         self.pid = worker_pid
 
 NotHandled = 0
-Handled = 1
+Handled    = 1
 
 Registered        = 10
 AlreadyRegistered = 14
@@ -53,18 +60,45 @@ bb.utils._context["NotHandled"] = NotHandled
 bb.utils._context["Handled"] = Handled
 
 def fire_class_handlers(event, d):
+    if isinstance(event, logging.LogRecord):
+        return
+
     for handler in _handlers:
         h = _handlers[handler]
         event.data = d
         if type(h).__name__ == "code":
             locals = {"e": event}
             bb.utils.simple_exec(h, locals)
-            bb.utils.better_eval("tmpHandler(e)", locals)
+            ret = bb.utils.better_eval("tmpHandler(e)", locals)
+            if ret is not None:
+                warnings.warn("Using Handled/NotHandled in event handlers is deprecated",
+                              DeprecationWarning, stacklevel = 2)
         else:
             h(event)
         del event.data
 
+ui_queue = []
+@atexit.register
+def print_ui_queue():
+    """If we're exiting before a UI has been spawned, display any queued
+    LogRecords to the console."""
+    logger = logging.getLogger("BitBake")
+    if not _ui_handlers:
+        from bb.msg import BBLogFormatter
+        console = logging.StreamHandler(sys.stdout)
+        console.setFormatter(BBLogFormatter("%(levelname)s: %(message)s"))
+        logger.handlers = [console]
+        while ui_queue:
+            event = ui_queue.pop()
+            if isinstance(event, logging.LogRecord):
+                logger.handle(event)
+
 def fire_ui_handlers(event, d):
+    if not _ui_handlers:
+        # No UI handlers registered yet, queue up the messages
+        ui_queue.append(event)
+        return
+
     errors = []
     for h in _ui_handlers:
         #print "Sending event %s" % event
@@ -72,7 +106,7 @@ def fire_ui_handlers(event, d):
              # We use pickle here since it better handles object instances
              # which xmlrpc's marshaller does not. Events *must* be serializable
              # by pickle.
-            _ui_handlers[h].event.send((pickle.dumps(event)))
+            _ui_handlers[h].event.send(event)
         except:
             errors.append(h)
     for h in errors:
@@ -81,9 +115,9 @@ def fire_ui_handlers(event, d):
 def fire(event, d):
     """Fire off an Event"""
 
-    # We can fire class handlers in the worker process context and this is 
+    # We can fire class handlers in the worker process context and this is
     # desired so they get the task based datastore.
-    # UI handlers need to be fired in the server context so we defer this. They 
+    # UI handlers need to be fired in the server context so we defer this. They
     # don't have a datastore so the datastore context isn't a problem.
 
     fire_class_handlers(event, d)
@@ -94,15 +128,11 @@ def fire(event, d):
 
 def worker_fire(event, d):
     data = "<event>" + pickle.dumps(event) + "</event>"
-    try:
-        if os.write(worker_pipe, data) != len (data):
-            print "Error sending event to server (short write)"
-    except OSError:
-        sys.exit(1)
+    worker_pipe.write(data)
 
 def fire_from_worker(event, d):
     if not event.startswith("<event>") or not event.endswith("</event>"):
-        print "Error, not an event"
+        print("Error, not an event %s" % event)
         return
     event = pickle.loads(event[7:-8])
     fire_ui_handlers(event, d)
@@ -116,7 +146,7 @@ def register(name, handler):
 
     if handler is not None:
         # handle string containing python code
-        if type(handler).__name__ == "str":
+        if isinstance(handler, basestring):
             tmp = "def tmpHandler(e):\n%s" % handler
             comp = bb.utils.better_compile(tmp, "tmpHandler(e)", "bb.event._registerCode")
             _handlers[name] = comp
@@ -227,10 +257,11 @@ class BuildCompleted(BuildBase):
 class NoProvider(Event):
     """No Provider for an Event"""
 
-    def __init__(self, item, runtime=False):
+    def __init__(self, item, runtime=False, dependees=None):
         Event.__init__(self)
         self._item = item
         self._runtime = runtime
+        self._dependees = dependees
 
     def getItem(self):
         return self._item
@@ -265,10 +296,14 @@ class MultipleProviders(Event):
         """
         return self._candidates
 
-class ParseProgress(Event):
-    """
-    Parsing Progress Event
-    """
+class ParseStarted(Event):
+    """Recipe parsing for the runqueue has begun"""
+    def __init__(self, total):
+        Event.__init__(self)
+        self.total = total
+
+class ParseCompleted(Event):
+    """Recipe parsing for the runqueue has completed"""
 
     def __init__(self, cached, parsed, skipped, masked, virtuals, errors, total):
         Event.__init__(self)
@@ -281,6 +316,32 @@ class ParseProgress(Event):
         self.sofar = cached + parsed
         self.total = total
 
+class ParseProgress(Event):
+    """Recipe parsing progress"""
+
+    def __init__(self, current):
+        self.current = current
+
+class CacheLoadStarted(Event):
+    """Loading of the dependency cache has begun"""
+    def __init__(self, total):
+        Event.__init__(self)
+        self.total = total
+
+class CacheLoadProgress(Event):
+    """Cache loading progress"""
+    def __init__(self, current):
+        Event.__init__(self)
+        self.current = current
+
+class CacheLoadCompleted(Event):
+    """Cache loading is complete"""
+    def __init__(self, total, num_entries):
+        Event.__init__(self)
+        self.total = total
+        self.num_entries = num_entries
+
+
 class DepTreeGenerated(Event):
     """
     Event when a dependency tree has been generated
@@ -290,3 +351,41 @@ class DepTreeGenerated(Event):
         Event.__init__(self)
         self._depgraph = depgraph
 
+class MsgBase(Event):
+    """Base class for messages"""
+
+    def __init__(self, msg):
+        self._message = msg
+        Event.__init__(self)
+
+class MsgDebug(MsgBase):
+    """Debug Message"""
+
+class MsgNote(MsgBase):
+    """Note Message"""
+
+class MsgWarn(MsgBase):
+    """Warning Message"""
+
+class MsgError(MsgBase):
+    """Error Message"""
+
+class MsgFatal(MsgBase):
+    """Fatal Message"""
+
+class MsgPlain(MsgBase):
+    """General output"""
+
+class LogHandler(logging.Handler):
+    """Dispatch logging messages as bitbake events"""
+
+    def emit(self, record):
+        if record.exc_info:
+            lines = traceback.format_exception(*record.exc_info, limit=5)
+            record.msg += '\n%s' % ''.join(lines)
+            record.exc_info = None
+        fire(record, None)
+
+    def filter(self, record):
+        record.taskpid = worker_pid
+        return True
