@@ -24,10 +24,18 @@ BitBake build tools.
 #
 # Based on functions from the base bb module, Copyright 2003 Holger Schurig
 
+from __future__ import absolute_import
+from __future__ import print_function
 import os, re
+import logging
 import bb
 from   bb import data
 from   bb import persist_data
+from   bb import utils
+
+__version__ = "1"
+
+logger = logging.getLogger("BitBake.Fetch")
 
 class MalformedUrl(Exception):
     """Exception raised when encountering an invalid url"""
@@ -53,21 +61,6 @@ class InvalidSRCREV(Exception):
 def decodeurl(url):
     """Decodes an URL into the tokens (scheme, network location, path,
     user, password, parameters).
-
-    >>> decodeurl("http://www.google.com/index.html")
-    ('http', 'www.google.com', '/index.html', '', '', {})
-
-    CVS url with username, host and cvsroot. The cvs module to check out is in the
-    parameters:
-
-    >>> decodeurl("cvs://anoncvs@cvs.handhelds.org/cvs;module=familiar/dist/ipkg")
-    ('cvs', 'cvs.handhelds.org', '/cvs', 'anoncvs', '', {'module': 'familiar/dist/ipkg'})
-
-    Dito, but this time the username has a password part. And we also request a special tag
-    to check out.
-
-    >>> decodeurl("cvs://anoncvs:anonymous@cvs.handhelds.org/cvs;module=familiar/dist/ipkg;tag=V0-99-81")
-    ('cvs', 'cvs.handhelds.org', '/cvs', 'anoncvs', 'anonymous', {'tag': 'V0-99-81', 'module': 'familiar/dist/ipkg'})
     """
 
     m = re.compile('(?P<type>[^:]*)://((?P<user>.+)@)?(?P<location>[^;]+)(;(?P<parm>.*))?').match(url)
@@ -82,7 +75,7 @@ def decodeurl(url):
     parm = m.group('parm')
 
     locidx = location.find('/')
-    if locidx != -1:
+    if locidx != -1 and type.lower() != 'file':
         host = location[:locidx]
         path = location[locidx:]
     else:
@@ -100,7 +93,7 @@ def decodeurl(url):
     p = {}
     if parm:
         for s in parm.split(';'):
-            s1,s2 = s.split('=')
+            s1, s2 = s.split('=')
             p[s1] = s2
 
     return (type, host, path, user, pswd, p)
@@ -108,27 +101,12 @@ def decodeurl(url):
 def encodeurl(decoded):
     """Encodes a URL from tokens (scheme, network location, path,
     user, password, parameters).
-
-    >>> encodeurl(['http', 'www.google.com', '/index.html', '', '', {}])
-    'http://www.google.com/index.html'
-
-    CVS with username, host and cvsroot. The cvs module to check out is in the
-    parameters:
-
-    >>> encodeurl(['cvs', 'cvs.handhelds.org', '/cvs', 'anoncvs', '', {'module': 'familiar/dist/ipkg'}])
-    'cvs://anoncvs@cvs.handhelds.org/cvs;module=familiar/dist/ipkg'
-
-    Dito, but this time the username has a password part. And we also request a special tag
-    to check out.
-
-    >>> encodeurl(['cvs', 'cvs.handhelds.org', '/cvs', 'anoncvs', 'anonymous', {'tag': 'V0-99-81', 'module': 'familiar/dist/ipkg'}])
-    'cvs://anoncvs:anonymous@cvs.handhelds.org/cvs;tag=V0-99-81;module=familiar/dist/ipkg'
     """
 
     (type, host, path, user, pswd, p) = decoded
 
     if not type or not path:
-        bb.msg.fatal(bb.msg.domain.Fetcher, "invalid or missing parameters for url encoding")
+        raise MissingParameterError("Type or path url components missing when encoding %s" % decoded)
     url = '%s://' % type
     if user:
         url += "%s" % user
@@ -145,33 +123,26 @@ def encodeurl(decoded):
     return url
 
 def uri_replace(uri, uri_find, uri_replace, d):
-#   bb.msg.note(1, bb.msg.domain.Fetcher, "uri_replace: operating on %s" % uri)
     if not uri or not uri_find or not uri_replace:
-        bb.msg.debug(1, bb.msg.domain.Fetcher, "uri_replace: passed an undefined value, not replacing")
-    uri_decoded = list(bb.decodeurl(uri))
-    uri_find_decoded = list(bb.decodeurl(uri_find))
-    uri_replace_decoded = list(bb.decodeurl(uri_replace))
-    result_decoded = ['','','','','',{}]
+        logger.debug(1, "uri_replace: passed an undefined value, not replacing")
+    uri_decoded = list(decodeurl(uri))
+    uri_find_decoded = list(decodeurl(uri_find))
+    uri_replace_decoded = list(decodeurl(uri_replace))
+    result_decoded = ['', '', '', '', '', {}]
     for i in uri_find_decoded:
         loc = uri_find_decoded.index(i)
         result_decoded[loc] = uri_decoded[loc]
-        import types
-        if type(i) == types.StringType:
+        if isinstance(i, basestring):
             if (re.match(i, uri_decoded[loc])):
                 result_decoded[loc] = re.sub(i, uri_replace_decoded[loc], uri_decoded[loc])
                 if uri_find_decoded.index(i) == 2:
                     if d:
                         localfn = bb.fetch.localpath(uri, d)
                         if localfn:
-                            result_decoded[loc] = os.path.dirname(result_decoded[loc]) + "/" + os.path.basename(bb.fetch.localpath(uri, d))
-#                       bb.msg.note(1, bb.msg.domain.Fetcher, "uri_replace: matching %s against %s and replacing with %s" % (i, uri_decoded[loc], uri_replace_decoded[loc]))
+                            result_decoded[loc] = os.path.join(os.path.dirname(result_decoded[loc]), os.path.basename(bb.fetch.localpath(uri, d)))
             else:
-#               bb.msg.note(1, bb.msg.domain.Fetcher, "uri_replace: no match")
                 return uri
-#           else:
-#               for j in i:
-#                   FIXME: apply replacements against options
-    return bb.encodeurl(result_decoded)
+    return encodeurl(result_decoded)
 
 methods = []
 urldata_cache = {}
@@ -179,50 +150,45 @@ saved_headrevs = {}
 
 def fetcher_init(d):
     """
-    Called to initilize the fetchers once the configuration data is known
+    Called to initialize the fetchers once the configuration data is known.
     Calls before this must not hit the cache.
     """
-    pd = persist_data.PersistData(d)
-    # When to drop SCM head revisions controled by user policy
+    # When to drop SCM head revisions controlled by user policy
     srcrev_policy = bb.data.getVar('BB_SRCREV_POLICY', d, 1) or "clear"
     if srcrev_policy == "cache":
-        bb.msg.debug(1, bb.msg.domain.Fetcher, "Keeping SRCREV cache due to cache policy of: %s" % srcrev_policy)
+        logger.debug(1, "Keeping SRCREV cache due to cache policy of: %s", srcrev_policy)
     elif srcrev_policy == "clear":
-        bb.msg.debug(1, bb.msg.domain.Fetcher, "Clearing SRCREV cache due to cache policy of: %s" % srcrev_policy)
+        logger.debug(1, "Clearing SRCREV cache due to cache policy of: %s", srcrev_policy)
+        revs = persist_data.persist('BB_URI_HEADREVS', d)
         try:
-            bb.fetch.saved_headrevs = pd.getKeyValues("BB_URI_HEADREVS")
+            bb.fetch.saved_headrevs = revs.items()
         except:
             pass
-        pd.delDomain("BB_URI_HEADREVS")
+        revs.clear()
     else:
-        bb.msg.fatal(bb.msg.domain.Fetcher, "Invalid SRCREV cache policy of: %s" % srcrev_policy)
+        raise FetchError("Invalid SRCREV cache policy of: %s" % srcrev_policy)
 
     for m in methods:
         if hasattr(m, "init"):
             m.init(d)
 
-    # Make sure our domains exist
-    pd.addDomain("BB_URI_HEADREVS")
-    pd.addDomain("BB_URI_LOCALCOUNT")
-
-def fetcher_compare_revisons(d):
+def fetcher_compare_revisions(d):
     """
     Compare the revisions in the persistant cache with current values and
     return true/false on whether they've changed.
     """
 
-    pd = persist_data.PersistData(d)
-    data = pd.getKeyValues("BB_URI_HEADREVS")
+    data = persist_data.persist('BB_URI_HEADREVS', d).items()
     data2 = bb.fetch.saved_headrevs
 
     changed = False
     for key in data:
         if key not in data2 or data2[key] != data[key]:
-            bb.msg.debug(1, bb.msg.domain.Fetcher, "%s changed" % key)
+            logger.debug(1, "%s changed", key)
             changed = True
             return True
         else:
-            bb.msg.debug(2, bb.msg.domain.Fetcher, "%s did not change" % key)
+            logger.debug(2, "%s did not change", key)
     return False
 
 # Function call order is usually:
@@ -233,6 +199,7 @@ def fetcher_compare_revisons(d):
 
 def init(urls, d, setup = True):
     urldata = {}
+
     fn = bb.data.getVar('FILE', d, 1)
     if fn in urldata_cache:
         urldata = urldata_cache[fn]
@@ -244,11 +211,50 @@ def init(urls, d, setup = True):
     if setup:
         for url in urldata:
             if not urldata[url].setup:
-                urldata[url].setup_localpath(d) 
+                urldata[url].setup_localpath(d)
 
     urldata_cache[fn] = urldata
     return urldata
 
+def mirror_from_string(data):
+    return [ i.split() for i in (data or "").replace('\\n','\n').split('\n') if i ]
+
+def verify_checksum(u, ud, d):
+    """
+    verify the MD5 and SHA256 checksum for downloaded src
+
+    return value:
+        - True: checksum matched
+        - False: checksum unmatched
+
+    if checksum is missing in recipes file, "BB_STRICT_CHECKSUM" decide the return value.
+    if BB_STRICT_CHECKSUM = "1" then return false as unmatched, otherwise return true as
+    matched
+    """
+
+    if not ud.type in ["http", "https", "ftp", "ftps"]:
+        return
+
+    md5data = bb.utils.md5_file(ud.localpath)
+    sha256data = bb.utils.sha256_file(ud.localpath)
+
+    if (ud.md5_expected == None or ud.sha256_expected == None):
+        logger.warn('Missing SRC_URI checksum for %s, consider adding to the recipe:\n'
+                    'SRC_URI[%s] = "%s"\nSRC_URI[%s] = "%s"',
+                    ud.localpath, ud.md5_name, md5data,
+                    ud.sha256_name, sha256data)
+        if bb.data.getVar("BB_STRICT_CHECKSUM", d, True) == "1":
+            raise FetchError("No checksum specified for %s." % u)
+        return
+
+    if (ud.md5_expected != md5data or ud.sha256_expected != sha256data):
+        logger.error('The checksums for "%s" did not match.\n'
+                     '  MD5: expected "%s", got "%s"\n'
+                     '  SHA256: expected "%s", got "%s"\n',
+                     ud.localpath, ud.md5_expected, md5data,
+                     ud.sha256_expected, sha256data)
+        raise FetchError("%s checksum mismatch." % u)
+
 def go(d, urls = None):
     """
     Fetch all urls
@@ -261,65 +267,67 @@ def go(d, urls = None):
     for u in urls:
         ud = urldata[u]
         m = ud.method
-        if ud.localfile:
-            if not m.forcefetch(u, ud, d) and os.path.exists(ud.md5):
-                # File already present along with md5 stamp file
-                # Touch md5 file to show activity
-                try:
-                    os.utime(ud.md5, None)
-                except:
-                    # Errors aren't fatal here
-                    pass
-                continue
-            lf = bb.utils.lockfile(ud.lockfile)
-            if not m.forcefetch(u, ud, d) and os.path.exists(ud.md5):
-                # If someone else fetched this before we got the lock, 
-                # notice and don't try again
-                try:
-                    os.utime(ud.md5, None)
-                except:
-                    # Errors aren't fatal here
-                    pass
-                bb.utils.unlockfile(lf)
-                continue
+        localpath = ""
 
-        # First try fetching uri, u, from PREMIRRORS
-        mirrors = [ i.split() for i in (bb.data.getVar('PREMIRRORS', d, 1) or "").split('\n') if i ]
-        localpath = try_mirrors(d, u, mirrors)
-        if not localpath:
+        if not ud.localfile:
+            continue
+
+        lf = bb.utils.lockfile(ud.lockfile)
+
+        if m.try_premirror(u, ud, d):
+            # First try fetching uri, u, from PREMIRRORS
+            mirrors = mirror_from_string(bb.data.getVar('PREMIRRORS', d, True))
+            localpath = try_mirrors(d, u, mirrors, False, m.forcefetch(u, ud, d))
+        elif os.path.exists(ud.localfile):
+            localpath = ud.localfile
+
+        # Need to re-test forcefetch() which will return true if our copy is too old
+        if m.forcefetch(u, ud, d) or not localpath:
             # Next try fetching from the original uri, u
             try:
                 m.go(u, ud, d)
                 localpath = ud.localpath
             except FetchError:
+                # Remove any incomplete file
+                bb.utils.remove(ud.localpath)
                 # Finally, try fetching uri, u, from MIRRORS
-                mirrors = [ i.split() for i in (bb.data.getVar('MIRRORS', d, 1) or "").split('\n') if i ]
+                mirrors = mirror_from_string(bb.data.getVar('MIRRORS', d, True))
                 localpath = try_mirrors (d, u, mirrors)
                 if not localpath or not os.path.exists(localpath):
                     raise FetchError("Unable to fetch URL %s from any source." % u)
 
-        if localpath:
-            ud.localpath = localpath
+        ud.localpath = localpath
 
-        if ud.localfile:
-            if not m.forcefetch(u, ud, d):
-                Fetch.write_md5sum(u, ud, d)
-            bb.utils.unlockfile(lf)
+        if os.path.exists(ud.md5):
+            # Touch the md5 file to show active use of the download
+            try:
+                os.utime(ud.md5, None)
+            except:
+                # Errors aren't fatal here
+                pass
+        else:
+            # Only check the checksums if we've not seen this item before
+            verify_checksum(u, ud, d)
+            Fetch.write_md5sum(u, ud, d)
 
+        bb.utils.unlockfile(lf)
 
-def checkstatus(d):
+def checkstatus(d, urls = None):
     """
     Check all urls exist upstream
     init must have previously been called
     """
     urldata = init([], d, True)
 
-    for u in urldata:
+    if not urls:
+        urls = urldata
+
+    for u in urls:
         ud = urldata[u]
         m = ud.method
-        bb.msg.note(1, bb.msg.domain.Fetcher, "Testing URL %s" % u)
+        logger.debug(1, "Testing URL %s", u)
         # First try checking uri, u, from PREMIRRORS
-        mirrors = [ i.split() for i in (bb.data.getVar('PREMIRRORS', d, 1) or "").split('\n') if i ]
+        mirrors = mirror_from_string(bb.data.getVar('PREMIRRORS', d, True))
         ret = try_mirrors(d, u, mirrors, True)
         if not ret:
             # Next try checking from the original uri, u
@@ -327,11 +335,11 @@ def checkstatus(d):
                 ret = m.checkstatus(u, ud, d)
             except:
                 # Finally, try checking uri, u, from MIRRORS
-                mirrors = [ i.split() for i in (bb.data.getVar('MIRRORS', d, 1) or "").split('\n') if i ]
+                mirrors = mirror_from_string(bb.data.getVar('MIRRORS', d, True))
                 ret = try_mirrors (d, u, mirrors, True)
 
         if not ret:
-            bb.msg.fatal(bb.msg.domain.Fetcher, "URL %s doesn't work" % u)
+            raise FetchError("URL %s doesn't work" % u)
 
 def localpaths(d):
     """
@@ -341,27 +349,30 @@ def localpaths(d):
     urldata = init([], d, True)
 
     for u in urldata:
-        ud = urldata[u]      
+        ud = urldata[u]
         local.append(ud.localpath)
 
     return local
 
 srcrev_internal_call = False
 
+def get_autorev(d):
+    return get_srcrev(d)
+
 def get_srcrev(d):
     """
     Return the version string for the current package
     (usually to be used as PV)
     Most packages usually only have one SCM so we just pass on the call.
-    In the multi SCM case, we build a value based on SRCREV_FORMAT which must 
+    In the multi SCM case, we build a value based on SRCREV_FORMAT which must
     have been set.
     """
 
     #
-    # Ugly code alert. localpath in the fetchers will try to evaluate SRCREV which 
+    # Ugly code alert. localpath in the fetchers will try to evaluate SRCREV which
     # could translate into a call to here. If it does, we need to catch this
     # and provide some way so it knows get_srcrev is active instead of being
-    # some number etc. hence the srcrev_internal_call tracking and the magic  
+    # some number etc. hence the srcrev_internal_call tracking and the magic
     # "SRCREVINACTION" return value.
     #
     # Neater solutions welcome!
@@ -371,20 +382,21 @@ def get_srcrev(d):
 
     scms = []
 
-    # Only call setup_localpath on URIs which suppports_srcrev() 
+    # Only call setup_localpath on URIs which supports_srcrev()
     urldata = init(bb.data.getVar('SRC_URI', d, 1).split(), d, False)
     for u in urldata:
         ud = urldata[u]
-        if ud.method.suppports_srcrev():
+        if ud.method.supports_srcrev():
             if not ud.setup:
                 ud.setup_localpath(d)
             scms.append(u)
 
     if len(scms) == 0:
-        bb.msg.error(bb.msg.domain.Fetcher, "SRCREV was used yet no valid SCM was found in SRC_URI")
+        logger.error("SRCREV was used yet no valid SCM was found in SRC_URI")
         raise ParameterError
 
-    bb.data.setVar('__BB_DONT_CACHE','1', d)
+    if bb.data.getVar('BB_SRCREV_POLICY', d, True) != "cache":
+        bb.data.setVar('__BB_DONT_CACHE', '1', d)
 
     if len(scms) == 1:
         return urldata[scms[0]].method.sortable_revision(scms[0], urldata[scms[0]], d)
@@ -394,7 +406,7 @@ def get_srcrev(d):
     #
     format = bb.data.getVar('SRCREV_FORMAT', d, 1)
     if not format:
-        bb.msg.error(bb.msg.domain.Fetcher, "The SRCREV_FORMAT variable must be set when multiple SCMs are used.")
+        logger.error("The SRCREV_FORMAT variable must be set when multiple SCMs are used.")
         raise ParameterError
 
     for scm in scms:
@@ -407,7 +419,7 @@ def get_srcrev(d):
 
 def localpath(url, d, cache = True):
     """
-    Called from the parser with cache=False since the cache isn't ready 
+    Called from the parser with cache=False since the cache isn't ready
     at this point. Also called from classed in OE e.g. patch.bbclass
     """
     ud = init([url], d)
@@ -426,28 +438,31 @@ def runfetchcmd(cmd, d, quiet = False):
     # rather than host provided
     # Also include some other variables.
     # FIXME: Should really include all export varaiables?
-    exportvars = ['PATH', 'GIT_PROXY_COMMAND', 'GIT_PROXY_HOST', 'GIT_PROXY_PORT', 'GIT_CONFIG', 'http_proxy', 'ftp_proxy', 'SSH_AUTH_SOCK', 'SSH_AGENT_PID', 'HOME']
+    exportvars = ['PATH', 'GIT_PROXY_COMMAND', 'GIT_PROXY_HOST',
+                  'GIT_PROXY_PORT', 'GIT_CONFIG', 'http_proxy', 'ftp_proxy',
+                  'https_proxy', 'no_proxy', 'ALL_PROXY', 'all_proxy',
+                  'KRB5CCNAME', 'SSH_AUTH_SOCK', 'SSH_AGENT_PID', 'HOME']
 
     for var in exportvars:
         val = data.getVar(var, d, True)
         if val:
-            cmd = 'export ' + var + '=%s; %s' % (val, cmd)
+            cmd = 'export ' + var + '=\"%s\"; %s' % (val, cmd)
 
-    bb.msg.debug(1, bb.msg.domain.Fetcher, "Running %s" % cmd)
+    logger.debug(1, "Running %s", cmd)
 
     # redirect stderr to stdout
     stdout_handle = os.popen(cmd + " 2>&1", "r")
     output = ""
 
-    while 1:
+    while True:
         line = stdout_handle.readline()
         if not line:
             break
         if not quiet:
-            print line,
+            print(line, end=' ')
         output += line
 
-    status =  stdout_handle.close() or 0
+    status = stdout_handle.close() or 0
     signal = status >> 8
     exitstatus = status & 0xff
 
@@ -458,7 +473,7 @@ def runfetchcmd(cmd, d, quiet = False):
 
     return output
 
-def try_mirrors(d, uri, mirrors, check = False):
+def try_mirrors(d, uri, mirrors, check = False, force = False):
     """
     Try to use a mirrored version of the sources.
     This method will be automatically called before the fetchers go.
@@ -468,8 +483,8 @@ def try_mirrors(d, uri, mirrors, check = False):
     mirrors is the list of mirrors we're going to try
     """
     fpath = os.path.join(data.getVar("DL_DIR", d, 1), os.path.basename(uri))
-    if not check and os.access(fpath, os.R_OK):
-        bb.msg.debug(1, bb.msg.domain.Fetcher, "%s already exists, skipping checkout." % fpath)
+    if not check and os.access(fpath, os.R_OK) and not force:
+        logger.debug(1, "%s already exists, skipping checkout.", fpath)
         return fpath
 
     ld = d.createCopy()
@@ -479,23 +494,26 @@ def try_mirrors(d, uri, mirrors, check = False):
             try:
                 ud = FetchData(newuri, ld)
             except bb.fetch.NoMethodError:
-                bb.msg.debug(1, bb.msg.domain.Fetcher, "No method for %s" % uri)
+                logger.debug(1, "No method for %s", uri)
                 continue
 
             ud.setup_localpath(ld)
 
             try:
                 if check:
-                    ud.method.checkstatus(newuri, ud, ld)
+                    found = ud.method.checkstatus(newuri, ud, ld)
+                    if found:
+                        return found
                 else:
                     ud.method.go(newuri, ud, ld)
-                return ud.localpath
+                    return ud.localpath
             except (bb.fetch.MissingParameterError,
                     bb.fetch.FetchError,
                     bb.fetch.MD5SumError):
                 import sys
                 (type, value, traceback) = sys.exc_info()
-                bb.msg.debug(2, bb.msg.domain.Fetcher, "Mirror fetch failure: %s" % value)
+                logger.debug(2, "Mirror fetch failure: %s", value)
+                bb.utils.remove(ud.localpath)
                 continue
     return None
 
@@ -506,7 +524,7 @@ class FetchData(object):
     """
     def __init__(self, url, d):
         self.localfile = ""
-        (self.type, self.host, self.path, self.user, self.pswd, self.parm) = bb.decodeurl(data.expand(url, d))
+        (self.type, self.host, self.path, self.user, self.pswd, self.parm) = decodeurl(data.expand(url, d))
         self.date = Fetch.getSRCDate(self, d)
         self.url = url
         if not self.user and "user" in self.parm:
@@ -514,6 +532,16 @@ class FetchData(object):
         if not self.pswd and "pswd" in self.parm:
             self.pswd = self.parm["pswd"]
         self.setup = False
+
+        if "name" in self.parm:
+            self.md5_name = "%s.md5sum" % self.parm["name"]
+            self.sha256_name = "%s.sha256sum" % self.parm["name"]
+        else:
+            self.md5_name = "md5sum"
+            self.sha256_name = "sha256sum"
+        self.md5_expected = bb.data.getVarFlag("SRC_URI", self.md5_name, d)
+        self.sha256_expected = bb.data.getVarFlag("SRC_URI", self.sha256_name, d)
+
         for m in methods:
             if m.supports(url, self, d):
                 self.method = m
@@ -525,12 +553,13 @@ class FetchData(object):
         if "localpath" in self.parm:
             # if user sets localpath for file, use it instead.
             self.localpath = self.parm["localpath"]
+            self.basename = os.path.basename(self.localpath)
         else:
             premirrors = bb.data.getVar('PREMIRRORS', d, True)
             local = ""
             if premirrors and self.url:
                 aurl = self.url.split(";")[0]
-                mirrors = [ i.split() for i in (premirrors or "").split('\n') if i ]
+                mirrors = mirror_from_string(premirrors)
                 for (find, replace) in mirrors:
                     if replace.startswith("file://"):
                         path = aurl.split("://")[1]
@@ -549,10 +578,11 @@ class FetchData(object):
                 # Horrible...
                 bb.data.delVar("ISHOULDNEVEREXIST", d)
 
-        # Note: These files should always be in DL_DIR whereas localpath may not be.
-        basepath = bb.data.expand("${DL_DIR}/%s" % os.path.basename(self.localpath), d)
-        self.md5 = basepath + '.md5'
-        self.lockfile = basepath + '.lock'
+        if self.localpath is not None:
+            # Note: These files should always be in DL_DIR whereas localpath may not be.
+            basepath = bb.data.expand("${DL_DIR}/%s" % os.path.basename(self.localpath), d)
+            self.md5 = basepath + '.md5'
+            self.lockfile = basepath + '.lock'
 
 
 class Fetch(object):
@@ -570,10 +600,17 @@ class Fetch(object):
     def localpath(self, url, urldata, d):
         """
         Return the local filename of a given url assuming a successful fetch.
-        Can also setup variables in urldata for use in go (saving code duplication 
+        Can also setup variables in urldata for use in go (saving code duplication
         and duplicate code execution)
         """
         return url
+    def _strip_leading_slashes(self, relpath):
+        """
+        Remove leading slash as os.path.join can't cope
+        """
+        while os.path.isabs(relpath):
+            relpath = relpath[1:]
+        return relpath
 
     def setUrls(self, urls):
         self.__urls = urls
@@ -589,7 +626,7 @@ class Fetch(object):
         """
         return False
 
-    def suppports_srcrev(self):
+    def supports_srcrev(self):
         """
         The fetcher supports auto source revisions (SRCREV)
         """
@@ -602,12 +639,23 @@ class Fetch(object):
         """
         raise NoMethodError("Missing implementation for url")
 
+    def try_premirror(self, url, urldata, d):
+        """
+        Should premirrors be used?
+        """
+        if urldata.method.forcefetch(url, urldata, d):
+            return True
+        elif os.path.exists(urldata.md5) and os.path.exists(urldata.localfile):
+            return False
+        else:
+            return True
+
     def checkstatus(self, url, urldata, d):
         """
         Check the status of a URL
         Assumes localpath was called first
         """
-        bb.msg.note(1, bb.msg.domain.Fetcher, "URL %s could not be checked for status since no method exists." % url)
+        logger.info("URL %s could not be checked for status since no method exists.", url)
         return True
 
     def getSRCDate(urldata, d):
@@ -631,8 +679,8 @@ class Fetch(object):
         """
         Return:
             a) a source revision if specified
-	    b) True if auto srcrev is in action
-	    c) False otherwise
+            b) True if auto srcrev is in action
+            c) False otherwise
         """
 
         if 'rev' in ud.parm:
@@ -644,14 +692,18 @@ class Fetch(object):
         rev = None
         if 'name' in ud.parm:
             pn = data.getVar("PN", d, 1)
-            rev = data.getVar("SRCREV_pn-" + pn + "_" + ud.parm['name'], d, 1)
+            rev = data.getVar("SRCREV_%s_pn-%s" % (ud.parm['name'], pn), d, 1)
+            if not rev:
+                rev = data.getVar("SRCREV_pn-%s_%s" % (pn, ud.parm['name']), d, 1)
+            if not rev:
+                rev = data.getVar("SRCREV_%s" % (ud.parm['name']), d, 1)
         if not rev:
             rev = data.getVar("SRCREV", d, 1)
         if rev == "INVALID":
             raise InvalidSRCREV("Please set SRCREV to a valid value")
         if not rev:
             return False
-        if rev is "SRCREVINACTION":
+        if rev == "SRCREVINACTION":
             return True
         return rev
 
@@ -664,7 +716,7 @@ class Fetch(object):
             b) None otherwise
         """
 
-        localcount= None
+        localcount = None
         if 'name' in ud.parm:
             pn = data.getVar("PN", d, 1)
             localcount = data.getVar("LOCALCOUNT_" + ud.parm['name'], d, 1)
@@ -678,9 +730,7 @@ class Fetch(object):
         """
         Verify the md5sum we wanted with the one we got
         """
-        wanted_sum = None
-        if 'md5sum' in ud.parm:
-            wanted_sum = ud.parm['md5sum']
+        wanted_sum = ud.parm.get('md5sum')
         if not wanted_sum:
             return True
 
@@ -705,34 +755,33 @@ class Fetch(object):
         if not hasattr(self, "_latest_revision"):
             raise ParameterError
 
-        pd = persist_data.PersistData(d)
+        revs = persist_data.persist('BB_URI_HEADREVS', d)
         key = self.generate_revision_key(url, ud, d)
-        rev = pd.getValue("BB_URI_HEADREVS", key)
+        rev = revs[key]
         if rev != None:
             return str(rev)
 
-        rev = self._latest_revision(url, ud, d)
-        pd.setValue("BB_URI_HEADREVS", key, rev)
+        revs[key] = rev = self._latest_revision(url, ud, d)
         return rev
 
     def sortable_revision(self, url, ud, d):
         """
-        
+
         """
         if hasattr(self, "_sortable_revision"):
             return self._sortable_revision(url, ud, d)
 
-        pd = persist_data.PersistData(d)
+        localcounts = persist_data.persist('BB_URI_LOCALCOUNT', d)
         key = self.generate_revision_key(url, ud, d)
 
         latest_rev = self._build_revision(url, ud, d)
-        last_rev = pd.getValue("BB_URI_LOCALCOUNT", key + "_rev")
+        last_rev = localcounts[key + '_rev']
         uselocalcount = bb.data.getVar("BB_LOCALCOUNT_OVERRIDE", d, True) or False
         count = None
         if uselocalcount:
             count = Fetch.localcount_internal_helper(ud, d)
         if count is None:
-            count = pd.getValue("BB_URI_LOCALCOUNT", key + "_count")
+            count = localcounts[key + '_count']
 
         if last_rev == latest_rev:
             return str(count + "+" + latest_rev)
@@ -748,8 +797,8 @@ class Fetch(object):
         else:
             count = str(int(count) + 1)
 
-        pd.setValue("BB_URI_LOCALCOUNT", key + "_rev", latest_rev)
-        pd.setValue("BB_URI_LOCALCOUNT", key + "_count", count)
+        localcounts[key + '_rev'] = latest_rev
+        localcounts[key + '_count'] = count
 
         return str(count + "+" + latest_rev)
 
@@ -757,18 +806,18 @@ class Fetch(object):
         key = self._revision_key(url, ud, d)
         return "%s-%s" % (key, bb.data.getVar("PN", d, True) or "")
 
-import cvs
-import git
-import local
-import svn
-import wget
-import svk
-import ssh
-import perforce
-import bzr
-import hg
-import osc
-import repo
+from . import cvs
+from . import git
+from . import local
+from . import svn
+from . import wget
+from . import svk
+from . import ssh
+from . import perforce
+from . import bzr
+from . import hg
+from . import osc
+from . import repo
 
 methods.append(local.Local())
 methods.append(wget.Wget())
