@@ -27,6 +27,7 @@ BitBake build tools.
 
 import os
 import bb
+import bb.utils
 from   bb import data
 from   bb.fetch import Fetch
 
@@ -47,7 +48,7 @@ class Local(Fetch):
         if path[0] != "/":
             filespath = data.getVar('FILESPATH', d, 1)
             if filespath:
-                newpath = bb.which(filespath, path)
+                newpath = bb.utils.which(filespath, path)
             if not newpath:
                 filesdir = data.getVar('FILESDIR', d, 1)
                 if filesdir:
@@ -65,8 +66,8 @@ class Local(Fetch):
         Check the status of the url
         """
         if urldata.localpath.find("*") != -1:
-            bb.msg.note(1, bb.msg.domain.Fetcher, "URL %s looks like a glob and was therefore not checked." % url)	
-	    return True
+            logger.info("URL %s looks like a glob and was therefore not checked.", url)
+            return True
         if os.path.exists(urldata.localpath):
-           return True
+            return True
         return False
