@@ -22,9 +22,12 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 import re
+import logging
 from bb import data, utils
 import bb
 
+logger = logging.getLogger("BitBake.Provider")
+
 class NoProvider(Exception):
     """Exception raised when no provider of a build dependency can be found"""
 
@@ -62,7 +65,7 @@ def sortPriorities(pn, dataCache, pkg_pn = None):
 def preferredVersionMatch(pe, pv, pr, preferred_e, preferred_v, preferred_r):
     """
     Check if the version pe,pv,pr is the preferred one.
-    If there is preferred version defined and ends with '%', then pv has to start with that version after removing the '%' 
+    If there is preferred version defined and ends with '%', then pv has to start with that version after removing the '%'
     """
     if (pr == preferred_r or preferred_r == None):
         if (pe == preferred_e or preferred_e == None):
@@ -103,7 +106,7 @@ def findPreferredProvider(pn, cfgData, dataCache, pkg_pn = None, item = None):
 
         for file_set in pkg_pn:
             for f in file_set:
-                pe,pv,pr = dataCache.pkg_pepvpr[f]
+                pe, pv, pr = dataCache.pkg_pepvpr[f]
                 if preferredVersionMatch(pe, pv, pr, preferred_e, preferred_v, preferred_r):
                     preferred_file = f
                     preferred_ver = (pe, pv, pr)
@@ -120,9 +123,9 @@ def findPreferredProvider(pn, cfgData, dataCache, pkg_pn = None, item = None):
         if item:
             itemstr = " (for item %s)" % item
         if preferred_file is None:
-            bb.msg.note(1, bb.msg.domain.Provider, "preferred version %s of %s not available%s" % (pv_str, pn, itemstr))
+            logger.info("preferred version %s of %s not available%s", pv_str, pn, itemstr)
         else:
-            bb.msg.debug(1, bb.msg.domain.Provider, "selecting %s as PREFERRED_VERSION %s of package %s%s" % (preferred_file, pv_str, pn, itemstr))
+            logger.debug(1, "selecting %s as PREFERRED_VERSION %s of package %s%s", preferred_file, pv_str, pn, itemstr)
 
     return (preferred_ver, preferred_file)
 
@@ -136,7 +139,7 @@ def findLatestProvider(pn, cfgData, dataCache, file_set):
     latest_p = 0
     latest_f = None
     for file_name in file_set:
-        pe,pv,pr = dataCache.pkg_pepvpr[file_name]
+        pe, pv, pr = dataCache.pkg_pepvpr[file_name]
         dp = dataCache.pkg_dp[file_name]
 
         if (latest is None) or ((latest_p == dp) and (utils.vercmp(latest, (pe, pv, pr)) < 0)) or (dp > latest_p):
@@ -169,14 +172,14 @@ def findBestProvider(pn, cfgData, dataCache, pkg_pn = None, item = None):
 
 def _filterProviders(providers, item, cfgData, dataCache):
     """
-    Take a list of providers and filter/reorder according to the 
+    Take a list of providers and filter/reorder according to the
     environment variables and previous build results
     """
     eligible = []
     preferred_versions = {}
     sortpkg_pn = {}
 
-    # The order of providers depends on the order of the files on the disk 
+    # The order of providers depends on the order of the files on the disk
     # up to here. Sort pkg_pn to make dependency issues reproducible rather
     # than effectively random.
     providers.sort()
@@ -189,7 +192,7 @@ def _filterProviders(providers, item, cfgData, dataCache):
             pkg_pn[pn] = []
         pkg_pn[pn].append(p)
 
-    bb.msg.debug(1, bb.msg.domain.Provider, "providers for %s are: %s" % (item, pkg_pn.keys()))
+    logger.debug(1, "providers for %s are: %s", item, pkg_pn.keys())
 
     # First add PREFERRED_VERSIONS
     for pn in pkg_pn:
@@ -206,7 +209,7 @@ def _filterProviders(providers, item, cfgData, dataCache):
         eligible.append(preferred_versions[pn][1])
 
     if len(eligible) == 0:
-        bb.msg.error(bb.msg.domain.Provider, "no eligible providers for %s" % item)
+        logger.error("no eligible providers for %s", item)
         return 0
 
     # If pn == item, give it a slight default preference
@@ -226,7 +229,7 @@ def _filterProviders(providers, item, cfgData, dataCache):
 
 def filterProviders(providers, item, cfgData, dataCache):
     """
-    Take a list of providers and filter/reorder according to the 
+    Take a list of providers and filter/reorder according to the
     environment variables and previous build results
     Takes a "normal" target item
     """
@@ -242,19 +245,19 @@ def filterProviders(providers, item, cfgData, dataCache):
         for p in eligible:
             pn = dataCache.pkg_fn[p]
             if dataCache.preferred[item] == pn:
-                bb.msg.note(2, bb.msg.domain.Provider, "selecting %s to satisfy %s due to PREFERRED_PROVIDERS" % (pn, item))
+                logger.verbose("selecting %s to satisfy %s due to PREFERRED_PROVIDERS", pn, item)
                 eligible.remove(p)
                 eligible = [p] + eligible
                 foundUnique = True
                 break
 
-    bb.msg.debug(1, bb.msg.domain.Provider, "sorted providers for %s are: %s" % (item, eligible))
+    logger.debug(1, "sorted providers for %s are: %s", item, eligible)
 
     return eligible, foundUnique
 
 def filterProvidersRunTime(providers, item, cfgData, dataCache):
     """
-    Take a list of providers and filter/reorder according to the 
+    Take a list of providers and filter/reorder according to the
     environment variables and previous build results
     Takes a "runtime" target item
     """
@@ -268,11 +271,11 @@ def filterProvidersRunTime(providers, item, cfgData, dataCache):
         pn = dataCache.pkg_fn[p]
         provides = dataCache.pn_provides[pn]
         for provide in provides:
-            bb.msg.note(2, bb.msg.domain.Provider, "checking PREFERRED_PROVIDER_%s" % (provide))
+            logger.verbose("checking PREFERRED_PROVIDER_%s", provide)
             prefervar = bb.data.getVar('PREFERRED_PROVIDER_%s' % provide, cfgData, 1)
             if prefervar == pn:
                 var = "PREFERRED_PROVIDER_%s = %s" % (provide, prefervar)
-                bb.msg.note(2, bb.msg.domain.Provider, "selecting %s to satisfy runtime %s due to %s" % (pn, item, var))
+                logger.verbose("selecting %s to satisfy runtime %s due to %s", pn, item, var)
                 preferred_vars.append(var)
                 eligible.remove(p)
                 eligible = [p] + eligible
@@ -282,9 +285,9 @@ def filterProvidersRunTime(providers, item, cfgData, dataCache):
     numberPreferred = len(preferred)
 
     if numberPreferred > 1:
-        bb.msg.error(bb.msg.domain.Provider, "Conflicting PREFERRED_PROVIDER entries were found which resulted in an attempt to select multiple providers (%s) for runtime dependecy %s\nThe entries resulting in this conflict were: %s" % (preferred, item, preferred_vars))
+        logger.error("Conflicting PREFERRED_PROVIDER entries were found which resulted in an attempt to select multiple providers (%s) for runtime dependecy %s\nThe entries resulting in this conflict were: %s", preferred, item, preferred_vars)
 
-    bb.msg.debug(1, bb.msg.domain.Provider, "sorted providers for %s are: %s" % (item, eligible))
+    logger.debug(1, "sorted providers for %s are: %s", item, eligible)
 
     return eligible, numberPreferred
 
@@ -297,7 +300,7 @@ def getRuntimeProviders(dataCache, rdepend):
     rproviders = []
 
     if rdepend in dataCache.rproviders:
-       rproviders += dataCache.rproviders[rdepend]
+        rproviders += dataCache.rproviders[rdepend]
 
     if rdepend in dataCache.packages:
         rproviders += dataCache.packages[rdepend]
@@ -314,7 +317,7 @@ def getRuntimeProviders(dataCache, rdepend):
             try:
                 regexp = re.compile(pattern)
             except:
-                bb.msg.error(bb.msg.domain.Provider, "Error parsing re expression: %s" % pattern)
+                logger.error("Error parsing regular expression '%s'", pattern)
                 raise
             regexp_cache[pattern] = regexp
         if regexp.match(rdepend):
