@@ -19,10 +19,19 @@ BitBake Utility Functions
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-separators = ".-"
-
-import re, fcntl, os, types, bb, string, stat, shutil, time
+import re, fcntl, os, string, stat, shutil, time
+import sys
+import errno
+import logging
+import bb
+import bb.msg
 from commands import getstatusoutput
+from contextlib import contextmanager
+
+logger = logging.getLogger("BitBake.Util")
+
+# Version comparison
+separators = ".-"
 
 # Context used in better_exec, eval
 _context = {
@@ -67,9 +76,9 @@ def vercmp_part(a, b):
         if ca == None and cb == None:
             return 0
 
-        if type(ca) is types.StringType:
+        if isinstance(ca, basestring):
             sa = ca in separators
-        if type(cb) is types.StringType:
+        if isinstance(cb, basestring):
             sb = cb in separators
         if sa and not sb:
             return -1
@@ -85,33 +94,26 @@ def vercmp(ta, tb):
     (ea, va, ra) = ta
     (eb, vb, rb) = tb
 
-    r = int(ea)-int(eb)
+    r = int(ea or 0) - int(eb or 0)
     if (r == 0):
         r = vercmp_part(va, vb)
     if (r == 0):
         r = vercmp_part(ra, rb)
     return r
 
-_package_weights_ = {"pre":-2,"p":0,"alpha":-4,"beta":-3,"rc":-1}    # dicts are unordered
-_package_ends_    = ["pre", "p", "alpha", "beta", "rc", "cvs", "bk", "HEAD" ]            # so we need ordered list
+_package_weights_ = {"pre":-2, "p":0, "alpha":-4, "beta":-3, "rc":-1}    # dicts are unordered
+_package_ends_ = ["pre", "p", "alpha", "beta", "rc", "cvs", "bk", "HEAD" ]            # so we need ordered list
 
 def relparse(myver):
     """Parses the last elements of a version number into a triplet, that can
-    later be compared:
-
-    >>> relparse('1.2_pre3')
-    [1.2, -2, 3.0]
-    >>> relparse('1.2b')
-    [1.2, 98, 0]
-    >>> relparse('1.2')
-    [1.2, 0, 0]
+    later be compared.
     """
 
-    number   = 0
-    p1       = 0
-    p2       = 0
+    number = 0
+    p1 = 0
+    p2 = 0
     mynewver = myver.split('_')
-    if len(mynewver)==2:
+    if len(mynewver) == 2:
         # an _package_weights_
         number = float(mynewver[0])
         match = 0
@@ -139,40 +141,29 @@ def relparse(myver):
         divider = len(myver)-1
         if myver[divider:] not in "1234567890":
             #letter at end
-            p1     = ord(myver[divider:])
+            p1 = ord(myver[divider:])
             number = float(myver[0:divider])
         else:
             number = float(myver)
-    return [number,p1,p2]
+    return [number, p1, p2]
 
 __vercmp_cache__ = {}
 
-def vercmp_string(val1,val2):
+def vercmp_string(val1, val2):
     """This takes two version strings and returns an integer to tell you whether
     the versions are the same, val1>val2 or val2>val1.
-
-    >>> vercmp('1', '2')
-    -1.0
-    >>> vercmp('2', '1')
-    1.0
-    >>> vercmp('1', '1.0')
-    0
-    >>> vercmp('1', '1.1')
-    -1.0
-    >>> vercmp('1.1', '1_p2')
-    1.0
     """
 
     # quick short-circuit
     if val1 == val2:
         return 0
-    valkey = val1+" "+val2
+    valkey = val1 + " " + val2
 
     # cache lookup
     try:
         return __vercmp_cache__[valkey]
         try:
-            return - __vercmp_cache__[val2+" "+val1]
+            return - __vercmp_cache__[val2 + " " + val1]
         except KeyError:
             pass
     except KeyError:
@@ -193,21 +184,21 @@ def vercmp_string(val1,val2):
     # replace '-' by '.'
     # FIXME: Is it needed? can val1/2 contain '-'?
 
-    val1 = string.split(val1,'-')
+    val1 = val1.split("-")
     if len(val1) == 2:
-        val1[0] = val1[0] +"."+ val1[1]
-    val2 = string.split(val2,'-')
+        val1[0] = val1[0] + "." + val1[1]
+    val2 = val2.split("-")
     if len(val2) == 2:
-        val2[0] = val2[0] +"."+ val2[1]
+        val2[0] = val2[0] + "." + val2[1]
 
-    val1 = string.split(val1[0],'.')
-    val2 = string.split(val2[0],'.')
+    val1 = val1[0].split('.')
+    val2 = val2[0].split('.')
 
     # add back decimal point so that .03 does not become "3" !
-    for x in range(1,len(val1)):
+    for x in xrange(1, len(val1)):
         if val1[x][0] == '0' :
             val1[x] = '.' + val1[x]
-    for x in range(1,len(val2)):
+    for x in xrange(1, len(val2)):
         if val2[x][0] == '0' :
             val2[x] = '.' + val2[x]
 
@@ -224,10 +215,10 @@ def vercmp_string(val1,val2):
         val2[-1] += '_' + val2_prepart
     # The above code will extend version numbers out so they
     # have the same number of digits.
-    for x in range(0,len(val1)):
+    for x in xrange(0, len(val1)):
         cmp1 = relparse(val1[x])
         cmp2 = relparse(val2[x])
-        for y in range(0,3):
+        for y in xrange(0, 3):
             myret = cmp1[y] - cmp2[y]
             if myret != 0:
                 __vercmp_cache__[valkey] = myret
@@ -263,10 +254,10 @@ def explode_dep_versions(s):
     """
     Take an RDEPENDS style string of format:
     "DEPEND1 (optional version) DEPEND2 (optional version) ..."
-    and return a dictonary of dependencies and versions.
+    and return a dictionary of dependencies and versions.
     """
     r = {}
-    l = s.split()
+    l = s.replace(",", "").split()
     lastdep = None
     lastver = ""
     inversion = False
@@ -288,19 +279,30 @@ def explode_dep_versions(s):
 
     return r
 
+def join_deps(deps):
+    """
+    Take the result from explode_dep_versions and generate a dependency string
+    """
+    result = []
+    for dep in deps:
+        if deps[dep]:
+            result.append(dep + " (" + deps[dep] + ")")
+        else:
+            result.append(dep)
+    return ", ".join(result)
+
 def _print_trace(body, line):
     """
     Print the Environment of a Text Body
     """
-    import bb
-
     # print the environment of the method
-    bb.msg.error(bb.msg.domain.Util, "Printing the environment of the function")
-    min_line = max(1,line-4)
-    max_line = min(line+4,len(body)-1)
-    for i in range(min_line,max_line+1):
-        bb.msg.error(bb.msg.domain.Util, "\t%.4d:%s" % (i, body[i-1]) )
-
+    min_line = max(1, line-4)
+    max_line = min(line + 4, len(body))
+    for i in xrange(min_line, max_line + 1):
+        if line == i:
+            logger.error(' *** %.4d:%s', i, body[i-1])
+        else:
+            logger.error('     %.4d:%s', i, body[i-1])
 
 def better_compile(text, file, realfile, mode = "exec"):
     """
@@ -309,95 +311,93 @@ def better_compile(text, file, realfile, mode = "exec"):
     """
     try:
         return compile(text, file, mode)
-    except Exception, e:
-        import bb,sys
-
+    except Exception as e:
         # split the text into lines again
         body = text.split('\n')
-        bb.msg.error(bb.msg.domain.Util, "Error in compiling python function in: ", realfile)
-        bb.msg.error(bb.msg.domain.Util, "The lines leading to this error were:")
-        bb.msg.error(bb.msg.domain.Util, "\t%d:%s:'%s'" % (e.lineno, e.__class__.__name__, body[e.lineno-1]))
+        logger.error("Error in compiling python function in %s", realfile)
+        logger.error(str(e))
+        if e.lineno:
+            logger.error("The lines leading to this error were:")
+            logger.error("\t%d:%s:'%s'", e.lineno, e.__class__.__name__, body[e.lineno-1])
+            _print_trace(body, e.lineno)
+        else:
+            logger.error("The function causing this error was:")
+            for line in body:
+                logger.error(line)
 
-        _print_trace(body, e.lineno)
+        raise
 
-        # exit now
-        sys.exit(1)
-
-def better_exec(code, context, text, realfile):
+def better_exec(code, context, text, realfile = "<code>"):
     """
     Similiar to better_compile, better_exec will
     print the lines that are responsible for the
     error.
     """
-    import bb,sys
+    import bb.parse
+    if not hasattr(code, "co_filename"):
+        code = better_compile(code, realfile, realfile)
     try:
-        exec code in _context, context
-    except:
-        (t,value,tb) = sys.exc_info()
+        exec(code, _context, context)
+    except Exception:
+        (t, value, tb) = sys.exc_info()
 
         if t in [bb.parse.SkipPackage, bb.build.FuncFailed]:
             raise
 
-        # print the Header of the Error Message
-        bb.msg.error(bb.msg.domain.Util, "Error in executing python function in: %s" % realfile)
-        bb.msg.error(bb.msg.domain.Util, "Exception:%s Message:%s" % (t,value) )
-
-        # let us find the line number now
-        while tb.tb_next:
-            tb = tb.tb_next
-
         import traceback
-        line = traceback.tb_lineno(tb)
+        exception = traceback.format_exception_only(t, value)
+        logger.error('Error executing a python function in %s:\n%s',
+                     realfile, ''.join(exception))
+
+        # Strip 'us' from the stack (better_exec call)
+        tb = tb.tb_next
+
+        textarray = text.split('\n')
+        linefailed = traceback.tb_lineno(tb)
+
+        tbextract = traceback.extract_tb(tb)
+        tbformat = "\n".join(traceback.format_list(tbextract))
+        logger.error("The stack trace of python calls that resulted in this exception/failure was:")
+        for line in tbformat.split('\n'):
+            logger.error(line)
+
+        logger.error("The code that was being executed was:")
+        _print_trace(textarray, linefailed)
+        logger.error("(file: '%s', lineno: %s, function: %s)", tbextract[0][0], tbextract[0][1], tbextract[0][2])
+
+        # See if this is a function we constructed and has calls back into other functions in
+        # "text". If so, try and improve the context of the error by diving down the trace
+        level = 0
+        nexttb = tb.tb_next
+        while nexttb is not None:
+            if tbextract[level][0] == tbextract[level+1][0] and tbextract[level+1][2] == tbextract[level][0]:
+                _print_trace(textarray, tbextract[level+1][1])
+                logger.error("(file: '%s', lineno: %s, function: %s)", tbextract[level+1][0], tbextract[level+1][1], tbextract[level+1][2])
+            else:
+                 break
+            nexttb = tb.tb_next
+            level = level + 1
 
-        _print_trace( text.split('\n'), line )
-        
         raise
 
 def simple_exec(code, context):
-    exec code in _context, context
+    exec(code, _context, context)
 
 def better_eval(source, locals):
     return eval(source, _context, locals)
 
-def Enum(*names):
-   """
-   A simple class to give Enum support
-   """
+@contextmanager
+def fileslocked(files):
+    """Context manager for locking and unlocking file locks."""
+    locks = []
+    if files:
+        for lockfile in files:
+            locks.append(bb.utils.lockfile(lockfile))
 
-   assert names, "Empty enums are not supported"
+    yield
 
-   class EnumClass(object):
-      __slots__ = names
-      def __iter__(self):        return iter(constants)
-      def __len__(self):         return len(constants)
-      def __getitem__(self, i):  return constants[i]
-      def __repr__(self):        return 'Enum' + str(names)
-      def __str__(self):         return 'enum ' + str(constants)
-
-   class EnumValue(object):
-      __slots__ = ('__value')
-      def __init__(self, value): self.__value = value
-      Value = property(lambda self: self.__value)
-      EnumType = property(lambda self: EnumType)
-      def __hash__(self):        return hash(self.__value)
-      def __cmp__(self, other):
-         # C fans might want to remove the following assertion
-         # to make all enums comparable by ordinal value {;))
-         assert self.EnumType is other.EnumType, "Only values from the same enum are comparable"
-         return cmp(self.__value, other.__value)
-      def __invert__(self):      return constants[maximum - self.__value]
-      def __nonzero__(self):     return bool(self.__value)
-      def __repr__(self):        return str(names[self.__value])
-
-   maximum = len(names) - 1
-   constants = [None] * len(names)
-   for i, each in enumerate(names):
-      val = EnumValue(i)
-      setattr(EnumClass, each, val)
-      constants[i] = val
-   constants = tuple(constants)
-   EnumType = EnumClass()
-   return EnumType
+    for lock in locks:
+        bb.utils.unlockfile(lock)
 
 def lockfile(name):
     """
@@ -406,41 +406,44 @@ def lockfile(name):
     """
     path = os.path.dirname(name)
     if not os.path.isdir(path):
-        import bb, sys
-        bb.msg.error(bb.msg.domain.Util, "Error, lockfile path does not exist!: %s" % path)
+        logger.error("Lockfile destination directory '%s' does not exist", path)
+        sys.exit(1)
+
+    if not os.access(path, os.W_OK):
+        logger.error("Error, lockfile path is not writable!: %s" % path)
         sys.exit(1)
 
     while True:
         # If we leave the lockfiles lying around there is no problem
         # but we should clean up after ourselves. This gives potential
-        # for races though. To work around this, when we acquire the lock 
-        # we check the file we locked was still the lock file on disk. 
-        # by comparing inode numbers. If they don't match or the lockfile 
+        # for races though. To work around this, when we acquire the lock
+        # we check the file we locked was still the lock file on disk.
+        # by comparing inode numbers. If they don't match or the lockfile
         # no longer exists, we start again.
 
-        # This implementation is unfair since the last person to request the 
+        # This implementation is unfair since the last person to request the
         # lock is the most likely to win it.
 
         try:
-            lf = open(name, "a+")
-            fcntl.flock(lf.fileno(), fcntl.LOCK_EX)
-            statinfo = os.fstat(lf.fileno())
+            lf = open(name, 'a+')
+            fileno = lf.fileno()
+            fcntl.flock(fileno, fcntl.LOCK_EX)
+            statinfo = os.fstat(fileno)
             if os.path.exists(lf.name):
-               statinfo2 = os.stat(lf.name)
-               if statinfo.st_ino == statinfo2.st_ino:
-                   return lf
-            # File no longer exists or changed, retry
-            lf.close
-        except Exception, e:
+                statinfo2 = os.stat(lf.name)
+                if statinfo.st_ino == statinfo2.st_ino:
+                    return lf
+            lf.close()
+        except Exception:
             continue
 
 def unlockfile(lf):
     """
-    Unlock a file locked using lockfile()				
+    Unlock a file locked using lockfile()
     """
     os.unlink(lf.name)
     fcntl.flock(lf.fileno(), fcntl.LOCK_UN)
-    lf.close
+    lf.close()
 
 def md5_file(filename):
     """
@@ -452,7 +455,7 @@ def md5_file(filename):
     except ImportError:
         import md5
         m = md5.new()
-    
+
     for line in open(filename):
         m.update(line)
     return m.hexdigest()
@@ -474,12 +477,25 @@ def sha256_file(filename):
         s.update(line)
     return s.hexdigest()
 
-def preserved_envvars_list():
+def preserved_envvars_exported():
+    """Variables which are taken from the environment and placed in and exported
+    from the metadata"""
+    return [
+        'BB_TASKHASH',
+        'HOME',
+        'LOGNAME',
+        'PATH',
+        'PWD',
+        'SHELL',
+        'TERM',
+        'USER',
+        'USERNAME',
+    ]
+
+def preserved_envvars_exported_interactive():
+    """Variables which are taken from the environment and placed in and exported
+    from the metadata, for interactive tasks"""
     return [
-        'BBPATH',
-        'BB_PRESERVE_ENV',
-        'BB_ENV_WHITELIST',
-        'BB_ENV_EXTRAWHITE',
         'COLORTERM',
         'DBUS_SESSION_BUS_ADDRESS',
         'DESKTOP_SESSION',
@@ -489,45 +505,50 @@ def preserved_envvars_list():
         'GNOME_KEYRING_SOCKET',
         'GPG_AGENT_INFO',
         'GTK_RC_FILES',
-        'HOME',
-        'LANG',
-        'LOGNAME',
-        'PATH',
-        'PWD',
         'SESSION_MANAGER',
-        'SHELL',
+        'KRB5CCNAME',
         'SSH_AUTH_SOCK',
-        'TERM',
-        'USER',
-        'USERNAME',
-        '_',
         'XAUTHORITY',
         'XDG_DATA_DIRS',
         'XDG_SESSION_COOKIE',
     ]
 
+def preserved_envvars():
+    """Variables which are taken from the environment and placed in the metadata"""
+    v = [
+        'BBPATH',
+        'BB_PRESERVE_ENV',
+        'BB_ENV_WHITELIST',
+        'BB_ENV_EXTRAWHITE',
+        'LANG',
+        '_',
+    ]
+    return v + preserved_envvars_exported() + preserved_envvars_exported_interactive()
+
 def filter_environment(good_vars):
     """
     Create a pristine environment for bitbake. This will remove variables that
     are not known and may influence the build in a negative way.
     """
 
-    import bb
-
     removed_vars = []
     for key in os.environ.keys():
         if key in good_vars:
             continue
-        
+
         removed_vars.append(key)
         os.unsetenv(key)
         del os.environ[key]
 
     if len(removed_vars):
-        bb.debug(1, "Removed the following variables from the environment:", ",".join(removed_vars))
+        logger.debug(1, "Removed the following variables from the environment: %s", ", ".join(removed_vars))
 
     return removed_vars
 
+def create_interactive_env(d):
+    for k in preserved_envvars_exported_interactive():
+        os.setenv(k, bb.data.getVar(k, d, True))
+
 def clean_environment():
     """
     Clean up any spurious environment variables. This will remove any
@@ -537,7 +558,7 @@ def clean_environment():
         if 'BB_ENV_WHITELIST' in os.environ:
             good_vars = os.environ['BB_ENV_WHITELIST'].split()
         else:
-            good_vars = preserved_envvars_list()
+            good_vars = preserved_envvars()
         if 'BB_ENV_EXTRAWHITE' in os.environ:
             good_vars.extend(os.environ['BB_ENV_EXTRAWHITE'].split())
         filter_environment(good_vars)
@@ -554,16 +575,30 @@ def build_environment(d):
     """
     Build an environment from all exported variables.
     """
-    import bb
+    import bb.data
     for var in bb.data.keys(d):
         export = bb.data.getVarFlag(var, "export", d)
         if export:
             os.environ[var] = bb.data.getVar(var, d, True) or ""
 
+def remove(path, recurse=False):
+    """Equivalent to rm -f or rm -rf"""
+    if not path:
+        return
+    import os, errno, shutil, glob
+    for name in glob.glob(path):
+        try:
+            os.unlink(name)
+        except OSError as exc:
+            if recurse and exc.errno == errno.EISDIR:
+                shutil.rmtree(name)
+            elif exc.errno != errno.ENOENT:
+                raise
+
 def prunedir(topdir):
     # Delete everything reachable from the directory named in 'topdir'.
     # CAUTION:  This is dangerous!
-    for root, dirs, files in os.walk(topdir, topdown=False):
+    for root, dirs, files in os.walk(topdir, topdown = False):
         for name in files:
             os.remove(os.path.join(root, name))
         for name in dirs:
@@ -578,181 +613,182 @@ def prunedir(topdir):
 # but thats possibly insane and suffixes is probably going to be small
 #
 def prune_suffix(var, suffixes, d):
-    # See if var ends with any of the suffixes listed and 
+    # See if var ends with any of the suffixes listed and
     # remove it if found
     for suffix in suffixes:
         if var.endswith(suffix):
             return var.replace(suffix, "")
     return var
 
-def mkdirhier(dir):
+def mkdirhier(directory):
     """Create a directory like 'mkdir -p', but does not complain if
     directory already exists like os.makedirs
     """
 
-    bb.debug(3, "mkdirhier(%s)" % dir)
     try:
-        os.makedirs(dir)
-        bb.debug(2, "created " + dir)
-    except OSError, e:
-        if e.errno != 17: raise e
+        os.makedirs(directory)
+    except OSError as e:
+        if e.errno != errno.EEXIST:
+            raise e
 
-import stat
-
-def movefile(src,dest,newmtime=None,sstat=None):
+def movefile(src, dest, newmtime = None, sstat = None):
     """Moves a file from src to dest, preserving all permissions and
     attributes; mtime will be preserved even when moving across
     filesystems.  Returns true on success and false on failure. Move is
     atomic.
     """
 
-    #print "movefile("+src+","+dest+","+str(newmtime)+","+str(sstat)+")"
+    #print "movefile(" + src + "," + dest + "," + str(newmtime) + "," + str(sstat) + ")"
     try:
         if not sstat:
-            sstat=os.lstat(src)
-    except Exception, e:
-        print "movefile: Stating source file failed...", e
+            sstat = os.lstat(src)
+    except Exception as e:
+        print("movefile: Stating source file failed...", e)
         return None
 
-    destexists=1
+    destexists = 1
     try:
-        dstat=os.lstat(dest)
+        dstat = os.lstat(dest)
     except:
-        dstat=os.lstat(os.path.dirname(dest))
-        destexists=0
+        dstat = os.lstat(os.path.dirname(dest))
+        destexists = 0
 
     if destexists:
         if stat.S_ISLNK(dstat[stat.ST_MODE]):
             try:
                 os.unlink(dest)
-                destexists=0
-            except Exception, e:
+                destexists = 0
+            except Exception as e:
                 pass
 
     if stat.S_ISLNK(sstat[stat.ST_MODE]):
         try:
-            target=os.readlink(src)
+            target = os.readlink(src)
             if destexists and not stat.S_ISDIR(dstat[stat.ST_MODE]):
                 os.unlink(dest)
-            os.symlink(target,dest)
+            os.symlink(target, dest)
             #os.lchown(dest,sstat[stat.ST_UID],sstat[stat.ST_GID])
             os.unlink(src)
             return os.lstat(dest)
-        except Exception, e:
-            print "movefile: failed to properly create symlink:", dest, "->", target, e
+        except Exception as e:
+            print("movefile: failed to properly create symlink:", dest, "->", target, e)
             return None
 
-    renamefailed=1
-    if sstat[stat.ST_DEV]==dstat[stat.ST_DEV]:
+    renamefailed = 1
+    if sstat[stat.ST_DEV] == dstat[stat.ST_DEV]:
         try:
-            ret=os.rename(src,dest)
-            renamefailed=0
-        except Exception, e:
-            import errno
-            if e[0]!=errno.EXDEV:
+            os.rename(src, dest)
+            renamefailed = 0
+        except Exception as e:
+            if e[0] != errno.EXDEV:
                 # Some random error.
-                print "movefile: Failed to move", src, "to", dest, e
+                print("movefile: Failed to move", src, "to", dest, e)
                 return None
             # Invalid cross-device-link 'bind' mounted or actually Cross-Device
 
     if renamefailed:
-        didcopy=0
+        didcopy = 0
         if stat.S_ISREG(sstat[stat.ST_MODE]):
             try: # For safety copy then move it over.
-                shutil.copyfile(src,dest+"#new")
-                os.rename(dest+"#new",dest)
-                didcopy=1
-            except Exception, e:
-                print 'movefile: copy', src, '->', dest, 'failed.', e
+                shutil.copyfile(src, dest + "#new")
+                os.rename(dest + "#new", dest)
+                didcopy = 1
+            except Exception as e:
+                print('movefile: copy', src, '->', dest, 'failed.', e)
                 return None
         else:
             #we don't yet handle special, so we need to fall back to /bin/mv
-            a=getstatusoutput("/bin/mv -f "+"'"+src+"' '"+dest+"'")
-            if a[0]!=0:
-                print "movefile: Failed to move special file:" + src + "' to '" + dest + "'", a
+            a = getstatusoutput("/bin/mv -f " + "'" + src + "' '" + dest + "'")
+            if a[0] != 0:
+                print("movefile: Failed to move special file:" + src + "' to '" + dest + "'", a)
                 return None # failure
         try:
             if didcopy:
-                os.lchown(dest,sstat[stat.ST_UID],sstat[stat.ST_GID])
+                os.lchown(dest, sstat[stat.ST_UID], sstat[stat.ST_GID])
                 os.chmod(dest, stat.S_IMODE(sstat[stat.ST_MODE])) # Sticky is reset on chown
                 os.unlink(src)
-        except Exception, e:
-            print "movefile: Failed to chown/chmod/unlink", dest, e
+        except Exception as e:
+            print("movefile: Failed to chown/chmod/unlink", dest, e)
             return None
 
     if newmtime:
-        os.utime(dest,(newmtime,newmtime))
+        os.utime(dest, (newmtime, newmtime))
     else:
         os.utime(dest, (sstat[stat.ST_ATIME], sstat[stat.ST_MTIME]))
-        newmtime=sstat[stat.ST_MTIME]
+        newmtime = sstat[stat.ST_MTIME]
     return newmtime
 
-def copyfile(src,dest,newmtime=None,sstat=None):
+def copyfile(src, dest, newmtime = None, sstat = None):
     """
     Copies a file from src to dest, preserving all permissions and
     attributes; mtime will be preserved even when moving across
     filesystems.  Returns true on success and false on failure.
     """
-    #print "copyfile("+src+","+dest+","+str(newmtime)+","+str(sstat)+")"
+    #print "copyfile(" + src + "," + dest + "," + str(newmtime) + "," + str(sstat) + ")"
     try:
         if not sstat:
-            sstat=os.lstat(src)
-    except Exception, e:
-        print "copyfile: Stating source file failed...", e
+            sstat = os.lstat(src)
+    except Exception as e:
+        print("copyfile: Stating source file failed...", e)
         return False
 
-    destexists=1
+    destexists = 1
     try:
-        dstat=os.lstat(dest)
+        dstat = os.lstat(dest)
     except:
-        dstat=os.lstat(os.path.dirname(dest))
-        destexists=0
+        dstat = os.lstat(os.path.dirname(dest))
+        destexists = 0
 
     if destexists:
         if stat.S_ISLNK(dstat[stat.ST_MODE]):
             try:
                 os.unlink(dest)
-                destexists=0
-            except Exception, e:
+                destexists = 0
+            except Exception as e:
                 pass
 
     if stat.S_ISLNK(sstat[stat.ST_MODE]):
         try:
-            target=os.readlink(src)
+            target = os.readlink(src)
             if destexists and not stat.S_ISDIR(dstat[stat.ST_MODE]):
                 os.unlink(dest)
-            os.symlink(target,dest)
+            os.symlink(target, dest)
             #os.lchown(dest,sstat[stat.ST_UID],sstat[stat.ST_GID])
             return os.lstat(dest)
-        except Exception, e:
-            print "copyfile: failed to properly create symlink:", dest, "->", target, e
+        except Exception as e:
+            print("copyfile: failed to properly create symlink:", dest, "->", target, e)
             return False
 
     if stat.S_ISREG(sstat[stat.ST_MODE]):
-            try: # For safety copy then move it over.
-                shutil.copyfile(src,dest+"#new")
-                os.rename(dest+"#new",dest)
-            except Exception, e:
-                print 'copyfile: copy', src, '->', dest, 'failed.', e
-                return False
+        os.chmod(src, stat.S_IRUSR) # Make sure we can read it
+        try: # For safety copy then move it over.
+            shutil.copyfile(src, dest + "#new")
+            os.rename(dest + "#new", dest)
+        except Exception as e:
+            print('copyfile: copy', src, '->', dest, 'failed.', e)
+            return False
+        finally:
+            os.chmod(src, sstat[stat.ST_MODE])
+            os.utime(src, (sstat[stat.ST_ATIME], sstat[stat.ST_MTIME]))
+
     else:
-            #we don't yet handle special, so we need to fall back to /bin/mv
-            a=getstatusoutput("/bin/cp -f "+"'"+src+"' '"+dest+"'")
-            if a[0]!=0:
-                print "copyfile: Failed to copy special file:" + src + "' to '" + dest + "'", a
-                return False # failure
+        #we don't yet handle special, so we need to fall back to /bin/mv
+        a = getstatusoutput("/bin/cp -f " + "'" + src + "' '" + dest + "'")
+        if a[0] != 0:
+            print("copyfile: Failed to copy special file:" + src + "' to '" + dest + "'", a)
+            return False # failure
     try:
-        os.lchown(dest,sstat[stat.ST_UID],sstat[stat.ST_GID])
+        os.lchown(dest, sstat[stat.ST_UID], sstat[stat.ST_GID])
         os.chmod(dest, stat.S_IMODE(sstat[stat.ST_MODE])) # Sticky is reset on chown
-    except Exception, e:
-        print "copyfile: Failed to chown/chmod/unlink", dest, e
+    except Exception as e:
+        print("copyfile: Failed to chown/chmod/unlink", dest, e)
         return False
 
     if newmtime:
-        os.utime(dest,(newmtime,newmtime))
+        os.utime(dest, (newmtime, newmtime))
     else:
         os.utime(dest, (sstat[stat.ST_ATIME], sstat[stat.ST_MTIME]))
-        newmtime=sstat[stat.ST_MTIME]
+        newmtime = sstat[stat.ST_MTIME]
     return newmtime
 
 def which(path, item, direction = 0):
@@ -770,3 +806,18 @@ def which(path, item, direction = 0):
             return next
 
     return ""
+
+def init_logger(logger, verbose, debug, debug_domains):
+    """
+    Set verbosity and debug levels in the logger
+    """
+
+    if debug:
+        bb.msg.set_debug_level(debug)
+    elif verbose:
+        bb.msg.set_verbose(True)
+    else:
+        bb.msg.set_debug_level(0)
+
+    if debug_domains:
+        bb.msg.set_debug_domains(debug_domains)
